
import urllib.request
from typing import List

import feedparser
from feedparser.util import FeedParserDict

from .data import AbstractArticleEntry, BasicArticleEntry


base_url = 'http://export.arxiv.org/api/query?'



class ArXivExtractor:
    def __init__(self, nb_articles_each_turn: int=100):
        self._nb_articles_each_turn = nb_articles_each_turn

    def _raw_retrieve_articles_api(self, dts: str, start: int, max_results: int) -> FeedParserDict:
        query = 'search_query=submittedDate:[{dts:}0000+TO+{dts:}2359]&start={start:}&max_results={max_results:}&sortBy=submittedDate&sortOrder=ascending'.format(
            dts=dts,
            start=start,
            max_results=max_results
        )

        url = base_url + query

        response = urllib.request.urlopen(url).read()
        return feedparser.parse(response)

    def _partial_retrieve_articles(self, dts: str, start: int=0, max_results: int=None) -> List[AbstractArticleEntry]:
        if max_results is None:
            max_results = self._nb_articles_each_turn
        feed = self._raw_retrieve_articles_api(dts, start, max_results)

        article_entries = []
        for entry in feed.entries:
            article_entry = BasicArticleEntry.make_entry_from_feed(entry)
            article_entries.append(article_entry)

        return article_entries

    def retrieve_all_articles_given_date(self, dts: str) -> List[AbstractArticleEntry]:
        article_entries = []
        all_retrieved = False
        start_idx = 0
        while not all_retrieved:
            this_article_entries = self._partial_retrieve_articles(
                dts,
                start=start_idx,
                max_results=self._nb_articles_each_turn
            )
            article_entries += this_article_entries
            if len(this_article_entries) < self._nb_articles_each_turn:
                all_retrieved = True
            else:
                start_idx += self._nb_articles_each_turn
        return article_entries

    @property
    def nb_articles_each_turn(self) -> int:
        return self._nb_articles_each_turn

    @nb_articles_each_turn.setter
    def nb_articles_each_turn(self, val: int):
        self._nb_articles_each_turn = val




import unittest

from arxivextract.extraction import ArXivExtractor
from arxivextract.data import AbstractArticleEntry, BasicArticleEntry


class TestExtractor(unittest.TestCase):
    def setUp(self):
        self.extractor = ArXivExtractor(nb_articles_each_turn=100)

    def test_partial_extract_1(self):
        # test partial run
        results = self.extractor._partial_retrieve_articles('20250401', start=0, max_results=10)
        assert len(results) == 10
        assert isinstance(results[0], AbstractArticleEntry)
        assert isinstance(results[0], BasicArticleEntry)
        assert results[1].data_version == '2025-03-23'
        assert results[0].arxiv_code == '2504.02863v1'
        assert results[0].arxiv_url == 'http://arxiv.org/abs/2504.02863v1'
        assert results[0].pdf_url == 'http://arxiv.org/pdf/2504.02863v1'
        assert results[0].primary_category == 'cs.CL'

    def test_partial_extract_2(self):
        results = self.extractor._partial_retrieve_articles('20250401', start=10, max_results=20)
        assert len(results) == 20
        assert isinstance(results[5], AbstractArticleEntry)
        assert isinstance(results[15], BasicArticleEntry)
        assert results[10].data_version == '2025-03-23'
        assert results[0].arxiv_code == '2504.00306v1'
        assert results[0].arxiv_url == 'http://arxiv.org/abs/2504.00306v1'
        assert results[0].pdf_url == 'http://arxiv.org/pdf/2504.00306v1'
        assert results[0].primary_category == 'cs.LG'

    def test_full_extraction(self):
        # full extraction
        results = self.extractor.retrieve_all_articles_given_date('20250401')
        # assert len(results) == 985
        assert isinstance(results[85], AbstractArticleEntry)
        assert isinstance(results[315], BasicArticleEntry)
        assert results[100].primary_category == 'astro-ph.HE'

    def test_turnnb(self):
        self.extractor.nb_articles_each_turn = 23
        assert self.extractor.nb_articles_each_turn == 23

        self.extractor.nb_articles_each_turn = 106
        assert self.extractor.nb_articles_each_turn == 106

        self.extractor.nb_articles_each_turn = 100
        assert self.extractor.nb_articles_each_turn == 100


if __name__ == '__main__':
    unittest.main()




from abc import ABC, abstractmethod

from keybert import KeyBERT


class AbstractKeywordExtractor(ABC):
    @abstractmethod
    def extract_keywords(self, text: str) -> list[str]:
        pass


class KeywordBertKeywordExtractor(AbstractKeywordExtractor):
    def __init__(self, config: dict, embed_model='all-MiniLM-L6-v2'):
        self._keyword_model = KeyBERT(model=embed_model)
        self._config = config

    def extract_keywords(self, text: str) -> list[str]:
        return self._keyword_model.extract_keywords(text, **self._config)

    @property
    def keywordbertmodel(self) -> KeyBERT:
        return self._keyword_model


keyword_configs = {
    '2025-04-02-keyword': {
        'model': 'KeyBERT',
        'embed_model': 'all-MiniLM-L6-v2',
        'configs': {
            'keyphrase_ngram_range': (1, 3),
            'stop_words': 'english',
            'top_n': 5
        }
    },
    '2025-04-23-keyword': {
        'model': 'KeyBERT',
        'embed_model': 'allenai-specter',
        'configs': {
            'keyphrase_ngram_range': (1, 2),
            'stop_words': 'english',
            'top_n': 5
        }
    }
}


def make_keyword_extractor(version: str) -> AbstractKeywordExtractor:
    if version not in keyword_configs.keys():
        raise ValueError('Keyword version {} is not found.'.format(version))

    keyword_extraction_model = keyword_configs[version].get('model')
    if keyword_extraction_model is None:
        raise ValueError('NoneType error!')
    if keyword_extraction_model == 'KeyBERT':
        return KeywordBertKeywordExtractor(
            keyword_configs[version].get('configs'),
            embed_model=keyword_configs[version].get('embed_model', 'all-MiniLM-L6-v2')
        )
    else:
        raise ValueError('Unknown keyword extraction model: {}'.format(keyword_extraction_model))




import re
from typing import List
from abc import ABC, abstractmethod
from dataclasses import dataclass

from feedparser.util import FeedParserDict

from .keywords import AbstractKeywordExtractor


def grab_arxiv_code_from_link(url):
    match_obj = re.match(r'http[s]?://*arxiv.org/(abs|pdf)/([\d\w\.]+)', url)
    if match_obj is None:
        raise ValueError('Invalid arXiv link!')
    return match_obj[2]


class DataVersionError(Exception):
    def __init__(self, given_version, class_version):
        self.given_version = given_version
        self.class_version = class_version

    def __str__(self):
        return "Wrong data version. Given version: {given:}, but this class version: {correct:}".format(
            given=self.given_version,
            correct=self.class_version
        )


class AbstractArticleEntry(ABC):
    @abstractmethod
    def to_dict(self) -> dict:
        pass

    @property
    @abstractmethod
    def data_version(self) -> str:
        pass


@dataclass
class BasicArticleEntry(AbstractArticleEntry):
    arxiv_code: str
    title: str
    authors: List[str]
    publish_time: str
    abstract: str
    arxiv_url: str
    pdf_url: str
    primary_category: str
    tags: List[str]

    def to_dict(self) -> dict:
        return {
            'data_version': self.data_version,
            'arxiv_code': self.arxiv_code,
            'title': self.title,
            'authors': self.authors,
            'publish_time': self.publish_time,
            'abstract': self.abstract,
            'arxiv_url': self.arxiv_url,
            'pdf_url': self.pdf_url,
            'primary_category': self.primary_category,
            'tags': self.tags
        }

    @property
    def data_version(self) -> str:
        return '2025-03-23'

    @property
    def summary(self) -> str:
        return self.abstract

    @classmethod
    def make_entry_from_feed(cls, feed_entry: FeedParserDict) -> AbstractArticleEntry:
        title = feed_entry.title
        authors = [author.name for author in feed_entry.authors]
        publish_time = feed_entry.published
        abstract = feed_entry.summary.replace('\n', ' ')
        arxiv_url = feed_entry.link
        pdf_url = ''
        for links_item in feed_entry.links:
            if links_item.type == 'application/pdf':
                pdf_url = links_item.href
                break
        primary_category = feed_entry.arxiv_primary_category['term']
        tags = [item['term'] for item in feed_entry.tags]
        arxiv_code = grab_arxiv_code_from_link(arxiv_url)

        return BasicArticleEntry(
            arxiv_code=arxiv_code,
            title=title,
            authors=authors,
            publish_time=publish_time,
            abstract=abstract,
            arxiv_url=arxiv_url,
            pdf_url=pdf_url,
            primary_category=primary_category,
            tags=tags
        )

    @classmethod
    def make_entry_from_dict(cls, entry_dict: dict) -> AbstractArticleEntry:
        if 'data_version' not in entry_dict.keys():
            return BasicArticleEntry(**entry_dict)
        elif entry_dict['data_version'] != '2025-03-23':
            raise DataVersionError(entry_dict['data_version'], '2025-03-23')
        else:
            return BasicArticleEntry(**{k: v for k, v in entry_dict.items() if k != 'data_version'})


@dataclass
class ArticleEntryWithKeywords(BasicArticleEntry):
    keywords: List[str]

    def set_keywords(self, keywords: List[str]):
        self.keywords = keywords

    def to_dict(self) -> dict:
        return {
            'data_version': self.data_version,
            'arxiv_code': self.arxiv_code,
            'title': self.title,
            'authors': self.authors,
            'publish_time': self.publish_time,
            'abstract': self.abstract,
            'arxiv_url': self.arxiv_url,
            'pdf_url': self.pdf_url,
            'primary_category': self.primary_category,
            'tags': self.tags,
            'keywords': self.keywords
        }

    @property
    def data_version(self) -> str:
        return '2025-04-03'

    @classmethod
    def make_entry_from_feed(cls, feed_entry: FeedParserDict) -> AbstractArticleEntry:
        title = feed_entry.title
        authors = [author.name for author in feed_entry.authors]
        publish_time = feed_entry.published
        abstract = feed_entry.summary.replace('\n', ' ')
        arxiv_url = feed_entry.link
        pdf_url = ''
        for links_item in feed_entry.links:
            if links_item.type == 'application/pdf':
                pdf_url = links_item.href
                break
        primary_category = feed_entry.arxiv_primary_category['term']
        tags = [item['term'] for item in feed_entry.tags]
        arxiv_code = grab_arxiv_code_from_link(arxiv_url)
        keywords = []

        return ArticleEntryWithKeywords(
            arxiv_code=arxiv_code,
            title=title,
            authors=authors,
            publish_time=publish_time,
            abstract=abstract,
            arxiv_url=arxiv_url,
            pdf_url=pdf_url,
            primary_category=primary_category,
            tags=tags,
            keywords=[]
        )

    @classmethod
    def make_entry_from_dict(cls, entry_dict: dict) -> AbstractArticleEntry:
        if 'data_version' not in entry_dict.keys():
            return BasicArticleEntry(**entry_dict)
        elif entry_dict['data_version'] != '2025-04-03':
            raise DataVersionError(entry_dict['data_version'], '2025-04-03')
        else:
            return ArticleEntryWithKeywords(**{k: v for k, v in entry_dict.items() if k != 'data_version'})

    @classmethod
    def make_entry_by_adding_keywords(cls, entry: BasicArticleEntry, keyword_extractor: AbstractKeywordExtractor) -> AbstractArticleEntry:
        keyword_prob_pairs = keyword_extractor.extract_keywords(entry.abstract)
        entry_dict = entry.to_dict()
        entry_dict['keywords'] = [keyword for keyword, _ in keyword_prob_pairs]
        entry_dict['data_version'] = '2025-04-03'
        return ArticleEntryWithKeywords.make_entry_from_dict(entry_dict)




import unittest

from arxivextract.extraction import ArXivExtractor
from arxivextract.data import AbstractArticleEntry, BasicArticleEntry, ArticleEntryWithKeywords
from arxivextract.keywords import make_keyword_extractor, AbstractKeywordExtractor, KeywordBertKeywordExtractor


class TestEntries(unittest.TestCase):
    def setUp(self):
        self.extractor = ArXivExtractor()

    def test_entry(self):
        results = self.extractor.retrieve_all_articles_given_date('20111231')
        assert len(results) == 117
        assert results[0].arxiv_code == '1201.0200v2'
        assert results[0].arxiv_url == 'http://arxiv.org/abs/1201.0200v2'
        assert results[0].pdf_url == 'http://arxiv.org/pdf/1201.0200v2'
        assert results[0].primary_category == 'quant-ph'
        assert results[0].data_version == '2025-03-23'

        result_dict = results[0].to_dict()
        assert isinstance(result_dict, dict)
        assert result_dict['data_version'] == '2025-03-23'
        assert 'quant-ph' in result_dict['tags']

    def test_dict_to_entry(self):
        dictentry = {
            'data_version': '2025-03-23',
            'arxiv_code': '1201.0200v2',
            'title': 'Gaussian matrix-product states for coding in bosonic communication\n  channels',
            'authors': ['Joachim Schäfer', 'Evgueni Karpov', 'Nicolas J. Cerf'],
            'publish_time': '2011-12-31T00:03:33Z',
            'abstract': 'The communication capacity of Gaussian bosonic channels with memory has recently attracted much interest. Here, we investigate a method to prepare the multimode entangled input symbol states for encoding classical information into these channels. In particular, we study the usefulness of a Gaussian matrix product state (GMPS) as an input symbol state, which can be sequentially generated although it remains heavily entangled for an arbitrary number of modes. We show that the GMPS can achieve more than 99.9% of the Gaussian capacity for Gaussian bosonic memory channels with a Markovian or non-Markovian correlated noise model in a large range of noise correlation strengths. Furthermore, we present a noise class for which the GMPS is the exact optimal input symbol state of the corresponding channel. Since GMPS are ground states of particular quadratic Hamiltonians, our results suggest a possible link between the theory of quantum communication channels and quantum many-body physics.',
            'arxiv_url': 'http://arxiv.org/abs/1201.0200v2',
            'pdf_url': 'http://arxiv.org/pdf/1201.0200v2',
            'primary_category': 'quant-ph',
            'tags': ['quant-ph']
        }
        entry = BasicArticleEntry.make_entry_from_dict(dictentry)
        assert isinstance(entry, BasicArticleEntry)
        assert isinstance(entry, AbstractArticleEntry)
        assert entry.title == dictentry['title']
        assert 'Evgueni Karpov' in entry.authors
        assert entry.data_version == '2025-03-23'

    def test_keyword_entry_from_dict(self):
        dictentry = {
            'data_version': '2025-03-23',
            'arxiv_code': '1201.0200v2',
            'title': 'Gaussian matrix-product states for coding in bosonic communication\n  channels',
            'authors': ['Joachim Schäfer', 'Evgueni Karpov', 'Nicolas J. Cerf'],
            'publish_time': '2011-12-31T00:03:33Z',
            'abstract': 'The communication capacity of Gaussian bosonic channels with memory has recently attracted much interest. Here, we investigate a method to prepare the multimode entangled input symbol states for encoding classical information into these channels. In particular, we study the usefulness of a Gaussian matrix product state (GMPS) as an input symbol state, which can be sequentially generated although it remains heavily entangled for an arbitrary number of modes. We show that the GMPS can achieve more than 99.9% of the Gaussian capacity for Gaussian bosonic memory channels with a Markovian or non-Markovian correlated noise model in a large range of noise correlation strengths. Furthermore, we present a noise class for which the GMPS is the exact optimal input symbol state of the corresponding channel. Since GMPS are ground states of particular quadratic Hamiltonians, our results suggest a possible link between the theory of quantum communication channels and quantum many-body physics.',
            'arxiv_url': 'http://arxiv.org/abs/1201.0200v2',
            'pdf_url': 'http://arxiv.org/pdf/1201.0200v2',
            'primary_category': 'quant-ph',
            'tags': ['quant-ph']
        }

        keyword_extractor = make_keyword_extractor('2025-04-02-keyword')
        assert isinstance(keyword_extractor, AbstractKeywordExtractor)
        assert isinstance(keyword_extractor, KeywordBertKeywordExtractor)
        basic_entry = BasicArticleEntry.make_entry_from_dict(dictentry)
        assert isinstance(basic_entry, AbstractArticleEntry)
        assert isinstance(basic_entry, BasicArticleEntry)
        assert basic_entry.data_version == '2025-03-23'
        keyword_entry = ArticleEntryWithKeywords.make_entry_by_adding_keywords(basic_entry, keyword_extractor)
        assert isinstance(keyword_entry, AbstractArticleEntry)
        assert isinstance(keyword_entry, ArticleEntryWithKeywords)
        assert isinstance(keyword_entry, BasicArticleEntry)
        assert keyword_entry.data_version == '2025-04-03'


if __name__ == '__main__':
    unittest.main()

